"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ValidityPeriodTypedDict(TypedDict):
    length_seconds: NotRequired[int]
    r"""The duration of the certificate's validity period, in seconds."""
    not_after: NotRequired[str]
    r"""An RFC-3339-formatted timestamp after which the certificate is no longer valid."""
    not_before: NotRequired[str]
    r"""An RFC-3339-formatted timestamp before which the certificate is not valid."""


class ValidityPeriod(BaseModel):
    length_seconds: Optional[int] = None
    r"""The duration of the certificate's validity period, in seconds."""

    not_after: Optional[str] = None
    r"""An RFC-3339-formatted timestamp after which the certificate is no longer valid."""

    not_before: Optional[str] = None
    r"""An RFC-3339-formatted timestamp before which the certificate is not valid."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PixelFormatTypedDict(TypedDict):
    big_endian: NotRequired[bool]
    r"""If pixel RGB data are in big-endian"""
    bits_per_pixel: NotRequired[int]
    r"""How many bits in a single full pixel datum. Valid values are: 8, 16, 32"""
    blue_max: NotRequired[int]
    r"""Max value of blue pixel"""
    blue_shift: NotRequired[int]
    r"""How many bits to right shift a pixel datum to get blue bits in lsb"""
    depth: NotRequired[int]
    r"""Color depth"""
    green_max: NotRequired[int]
    r"""Max value of green pixel"""
    green_shift: NotRequired[int]
    r"""How many bits to right shift a pixel datum to get green bits in lsb"""
    padding1: NotRequired[int]
    padding2: NotRequired[int]
    padding3: NotRequired[int]
    red_max: NotRequired[int]
    r"""Max value of red pixel"""
    red_shift: NotRequired[int]
    r"""How many bits to right shift a pixel datum to get red bits in lsb"""
    true_color: NotRequired[bool]
    r"""If false, color maps are used"""


class PixelFormat(BaseModel):
    big_endian: Optional[bool] = None
    r"""If pixel RGB data are in big-endian"""

    bits_per_pixel: Optional[int] = None
    r"""How many bits in a single full pixel datum. Valid values are: 8, 16, 32"""

    blue_max: Optional[int] = None
    r"""Max value of blue pixel"""

    blue_shift: Optional[int] = None
    r"""How many bits to right shift a pixel datum to get blue bits in lsb"""

    depth: Optional[int] = None
    r"""Color depth"""

    green_max: Optional[int] = None
    r"""Max value of green pixel"""

    green_shift: Optional[int] = None
    r"""How many bits to right shift a pixel datum to get green bits in lsb"""

    padding1: Optional[int] = None

    padding2: Optional[int] = None

    padding3: Optional[int] = None

    red_max: Optional[int] = None
    r"""Max value of red pixel"""

    red_shift: Optional[int] = None
    r"""How many bits to right shift a pixel datum to get red bits in lsb"""

    true_color: Optional[bool] = None
    r"""If false, color maps are used"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .hosttimelineevent import HostTimelineEvent, HostTimelineEventTypedDict
from censys_platform.types import BaseModel
from typing import Any, Dict
from typing_extensions import TypedDict


class HostTimelineEventAssetTypedDict(TypedDict):
    extensions: Dict[str, Any]
    resource: HostTimelineEventTypedDict


class HostTimelineEventAsset(BaseModel):
    extensions: Dict[str, Any]

    resource: HostTimelineEvent



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dsacryptographickey import DSACryptographicKey, DSACryptographicKeyTypedDict
from .ecdsacryptographickey import ECDSACryptographicKey, ECDSACryptographicKeyTypedDict
from .ed25519cryptographickey import (
    Ed25519CryptographicKey,
    Ed25519CryptographicKeyTypedDict,
)
from .rsacryptographickey import RSACryptographicKey, RSACryptographicKeyTypedDict
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SSHServerHostKeyTypedDict(TypedDict):
    certkey_public_key: NotRequired[str]
    dsa_public_key: NotRequired[DSACryptographicKeyTypedDict]
    ecdsa_public_key: NotRequired[ECDSACryptographicKeyTypedDict]
    ed25519_public_key: NotRequired[Ed25519CryptographicKeyTypedDict]
    fingerprint_sha256: NotRequired[str]
    rsa_public_key: NotRequired[RSACryptographicKeyTypedDict]


class SSHServerHostKey(BaseModel):
    certkey_public_key: Optional[str] = None

    dsa_public_key: Optional[DSACryptographicKey] = None

    ecdsa_public_key: Optional[ECDSACryptographicKey] = None

    ed25519_public_key: Optional[Ed25519CryptographicKey] = None

    fingerprint_sha256: Optional[str] = None

    rsa_public_key: Optional[RSACryptographicKey] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .capabilities import Capabilities, CapabilitiesTypedDict
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class OnvifServiceTypedDict(TypedDict):
    capabilities: NotRequired[CapabilitiesTypedDict]
    namespace: NotRequired[str]
    service_version_major: NotRequired[int]
    service_version_minor: NotRequired[int]
    xaddr: NotRequired[str]


class OnvifService(BaseModel):
    capabilities: Optional[Capabilities] = None

    namespace: Optional[str] = None

    service_version_major: Optional[int] = None

    service_version_minor: Optional[int] = None

    xaddr: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CrestronCp3TypedDict(TypedDict):
    version_string: NotRequired[str]


class CrestronCp3(BaseModel):
    version_string: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .dhcpdiscover_ipaddress import (
    DhcpdiscoverIPAddress,
    DhcpdiscoverIPAddressTypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DhcpdiscoverDeviceInfoTypedDict(TypedDict):
    alarm_input_channels: NotRequired[int]
    alarm_output_channels: NotRequired[int]
    device_class: NotRequired[str]
    device_id: NotRequired[str]
    device_type: NotRequired[str]
    http_port: NotRequired[int]
    ipv4_address: NotRequired[DhcpdiscoverIPAddressTypedDict]
    ipv6_address: NotRequired[DhcpdiscoverIPAddressTypedDict]
    machine_group: NotRequired[str]
    machine_name: NotRequired[str]
    manufacturer: NotRequired[str]
    port: NotRequired[int]
    remote_video_input_channels: NotRequired[int]
    serial_number: NotRequired[str]
    unlogin_func_mask: NotRequired[int]
    vendor: NotRequired[str]
    version: NotRequired[str]
    video_input_channels: NotRequired[int]
    video_output_channels: NotRequired[int]


class DhcpdiscoverDeviceInfo(BaseModel):
    alarm_input_channels: Optional[int] = None

    alarm_output_channels: Optional[int] = None

    device_class: Optional[str] = None

    device_id: Optional[str] = None

    device_type: Optional[str] = None

    http_port: Optional[int] = None

    ipv4_address: Optional[DhcpdiscoverIPAddress] = None

    ipv6_address: Optional[DhcpdiscoverIPAddress] = None

    machine_group: Optional[str] = None

    machine_name: Optional[str] = None

    manufacturer: Optional[str] = None

    port: Optional[int] = None

    remote_video_input_channels: Optional[int] = None

    serial_number: Optional[str] = None

    unlogin_func_mask: Optional[int] = None

    vendor: Optional[str] = None

    version: Optional[str] = None

    video_input_channels: Optional[int] = None

    video_output_channels: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PortmapV2EntryTypedDict(TypedDict):
    description: NotRequired[str]
    port: NotRequired[int]
    protocol: NotRequired[str]
    shorthand: NotRequired[str]
    version: NotRequired[int]


class PortmapV2Entry(BaseModel):
    description: Optional[str] = None

    port: Optional[int] = None

    protocol: Optional[str] = None

    shorthand: Optional[str] = None

    version: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .listcollectionsresponsev1 import (
    ListCollectionsResponseV1,
    ListCollectionsResponseV1TypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ResponseEnvelopeListCollectionsResponseV1TypedDict(TypedDict):
    result: NotRequired[ListCollectionsResponseV1TypedDict]


class ResponseEnvelopeListCollectionsResponseV1(BaseModel):
    result: Optional[ListCollectionsResponseV1] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .responseenvelopevaluecountsresponse import (
    ResponseEnvelopeValueCountsResponse,
    ResponseEnvelopeValueCountsResponseTypedDict,
)
from .searchvaluecountsinputbody import (
    SearchValueCountsInputBody,
    SearchValueCountsInputBodyTypedDict,
)
from censys_platform.types import BaseModel
from censys_platform.utils import FieldMetadata, QueryParamMetadata, RequestMetadata
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V3ThreathuntingValueCountsGlobalsTypedDict(TypedDict):
    organization_id: NotRequired[str]


class V3ThreathuntingValueCountsGlobals(BaseModel):
    organization_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class V3ThreathuntingValueCountsRequestTypedDict(TypedDict):
    search_value_counts_input_body: SearchValueCountsInputBodyTypedDict
    organization_id: NotRequired[str]
    r"""The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information."""


class V3ThreathuntingValueCountsRequest(BaseModel):
    search_value_counts_input_body: Annotated[
        SearchValueCountsInputBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    organization_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information."""


class V3ThreathuntingValueCountsResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: ResponseEnvelopeValueCountsResponseTypedDict


class V3ThreathuntingValueCountsResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: ResponseEnvelopeValueCountsResponse



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EtcdVersionTypedDict(TypedDict):
    cluster: NotRequired[str]
    server: NotRequired[str]


class EtcdVersion(BaseModel):
    cluster: Optional[str] = None

    server: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class TorServiceDescriptorTypedDict(TypedDict):
    algorithm_name: NotRequired[str]
    hash: NotRequired[str]
    hash_bits: NotRequired[int]
    onion: NotRequired[str]


class TorServiceDescriptor(BaseModel):
    algorithm_name: Optional[str] = None

    hash: Optional[str] = None

    hash_bits: Optional[int] = None

    onion: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .certificateasset import CertificateAsset, CertificateAssetTypedDict
from .hostassetwithmatchedservices import (
    HostAssetWithMatchedServices,
    HostAssetWithMatchedServicesTypedDict,
)
from .webpropertyasset import WebpropertyAsset, WebpropertyAssetTypedDict
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SearchQueryHitTypedDict(TypedDict):
    certificate_v1: NotRequired[CertificateAssetTypedDict]
    host_v1: NotRequired[HostAssetWithMatchedServicesTypedDict]
    webproperty_v1: NotRequired[WebpropertyAssetTypedDict]


class SearchQueryHit(BaseModel):
    certificate_v1: Optional[CertificateAsset] = None

    host_v1: Optional[HostAssetWithMatchedServices] = None

    webproperty_v1: Optional[WebpropertyAsset] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CiscoIpslaHandshakeHeaderTypedDict(TypedDict):
    length: NotRequired[int]
    seq: NotRequired[int]
    unknown: NotRequired[int]
    version: NotRequired[int]


class CiscoIpslaHandshakeHeader(BaseModel):
    length: Optional[int] = None

    seq: Optional[int] = None

    unknown: Optional[int] = None

    version: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class KubernetesNodeNodeAddressTypedDict(TypedDict):
    address: NotRequired[str]
    r"""Node address, IP/URL."""
    address_type: NotRequired[str]
    r"""Node address type, one of Hostname, ExternalIP or InternalIP."""


class KubernetesNodeNodeAddress(BaseModel):
    address: Optional[str] = None
    r"""Node address, IP/URL."""

    address_type: Optional[str] = None
    r"""Node address type, one of Hostname, ExternalIP or InternalIP."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .redis_rawoutput import RedisRawOutput, RedisRawOutputTypedDict
from censys_platform.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Dict, List, Optional
from typing_extensions import NotRequired, TypedDict


class RedisTypedDict(TypedDict):
    arch_bits: NotRequired[str]
    r"""The architecture bits (32 or 64) the Redis server used to build."""
    auth_response: NotRequired[str]
    r"""The response from the AUTH command, if sent."""
    build_id: NotRequired[str]
    r"""The Build ID of the Redis server."""
    commands: NotRequired[Nullable[List[str]]]
    r"""The list of commands actually sent to the server, serialized in inline format, like 'PING' or 'AUTH somePassword'."""
    commands_processed: NotRequired[int]
    r"""The total number of commands processed by the server."""
    connections_received: NotRequired[int]
    r"""The total number of connections accepted by the server."""
    gcc_version: NotRequired[str]
    r"""The version of the GCC compiler used to compile the Redis server."""
    git_sha1: NotRequired[str]
    r"""The Sha-1 Git commit hash the Redis server used."""
    info_response: NotRequired[Dict[str, str]]
    r"""The response from the INFO command. Should be a series of key:value pairs separated by CRLFs."""
    major: NotRequired[int]
    r"""Major is the version's major number."""
    mem_allocator: NotRequired[str]
    r"""The memory allocator."""
    minor: NotRequired[int]
    r"""Minor is the version's major number."""
    mode: NotRequired[str]
    r"""The mode the Redis server is running (standalone or cluster), read from the the info_response (if available)."""
    nonexistent_response: NotRequired[str]
    r"""The response from the NONEXISTENT command."""
    os: NotRequired[str]
    r"""The OS the Redis server is running, read from the the info_response (if available)."""
    patch_level: NotRequired[int]
    r"""Patchlevel is the version's patchlevel number."""
    ping_response: NotRequired[str]
    r"""The response from the PING command; should either be \"PONG\" or an authentication error."""
    quit_response: NotRequired[str]
    r"""The response to the QUIT command."""
    raw_command_output: NotRequired[Nullable[List[RedisRawOutputTypedDict]]]
    r"""The raw output returned by the server for each command sent; the indices match those of commands."""
    uptime: NotRequired[int]
    r"""The number of seconds since Redis server start."""
    used_memory: NotRequired[int]
    r"""The total number of bytes allocated by Redis using its allocator."""


class Redis(BaseModel):
    arch_bits: Optional[str] = None
    r"""The architecture bits (32 or 64) the Redis server used to build."""

    auth_response: Optional[str] = None
    r"""The response from the AUTH command, if sent."""

    build_id: Optional[str] = None
    r"""The Build ID of the Redis server."""

    commands: OptionalNullable[List[str]] = UNSET
    r"""The list of commands actually sent to the server, serialized in inline format, like 'PING' or 'AUTH somePassword'."""

    commands_processed: Optional[int] = None
    r"""The total number of commands processed by the server."""

    connections_received: Optional[int] = None
    r"""The total number of connections accepted by the server."""

    gcc_version: Optional[str] = None
    r"""The version of the GCC compiler used to compile the Redis server."""

    git_sha1: Optional[str] = None
    r"""The Sha-1 Git commit hash the Redis server used."""

    info_response: Optional[Dict[str, str]] = None
    r"""The response from the INFO command. Should be a series of key:value pairs separated by CRLFs."""

    major: Optional[int] = None
    r"""Major is the version's major number."""

    mem_allocator: Optional[str] = None
    r"""The memory allocator."""

    minor: Optional[int] = None
    r"""Minor is the version's major number."""

    mode: Optional[str] = None
    r"""The mode the Redis server is running (standalone or cluster), read from the the info_response (if available)."""

    nonexistent_response: Optional[str] = None
    r"""The response from the NONEXISTENT command."""

    os: Optional[str] = None
    r"""The OS the Redis server is running, read from the the info_response (if available)."""

    patch_level: Optional[int] = None
    r"""Patchlevel is the version's patchlevel number."""

    ping_response: Optional[str] = None
    r"""The response from the PING command; should either be \"PONG\" or an authentication error."""

    quit_response: Optional[str] = None
    r"""The response to the QUIT command."""

    raw_command_output: OptionalNullable[List[RedisRawOutput]] = UNSET
    r"""The raw output returned by the server for each command sent; the indices match those of commands."""

    uptime: Optional[int] = None
    r"""The number of seconds since Redis server start."""

    used_memory: Optional[int] = None
    r"""The total number of bytes allocated by Redis using its allocator."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "arch_bits",
            "auth_response",
            "build_id",
            "commands",
            "commands_processed",
            "connections_received",
            "gcc_version",
            "git_sha1",
            "info_response",
            "major",
            "mem_allocator",
            "minor",
            "mode",
            "nonexistent_response",
            "os",
            "patch_level",
            "ping_response",
            "quit_response",
            "raw_command_output",
            "uptime",
            "used_memory",
        ]
        nullable_fields = ["commands", "raw_command_output"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class IpmiCapabilitiesAuthStatusTypedDict(TypedDict):
    anonymous_login_enabled: NotRequired[bool]
    r"""If true, the server allows anonymous login."""
    auth_each_message: NotRequired[bool]
    r"""If true, each message must be authenticated."""
    has_anonymous_users: NotRequired[bool]
    r"""If true, the server has anonymous users."""
    has_named_users: NotRequired[bool]
    r"""If true, the server supports named users."""
    two_key_login_required: NotRequired[bool]
    r"""The KG field."""
    user_auth_disabled: NotRequired[bool]
    r"""If true, user authentication is disabled."""


class IpmiCapabilitiesAuthStatus(BaseModel):
    anonymous_login_enabled: Optional[bool] = None
    r"""If true, the server allows anonymous login."""

    auth_each_message: Optional[bool] = None
    r"""If true, each message must be authenticated."""

    has_anonymous_users: Optional[bool] = None
    r"""If true, the server has anonymous users."""

    has_named_users: Optional[bool] = None
    r"""If true, the server supports named users."""

    two_key_login_required: Optional[bool] = None
    r"""The KG field."""

    user_auth_disabled: Optional[bool] = None
    r"""If true, user authentication is disabled."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class UserTypedDict(TypedDict):
    auth_key_id: NotRequired[str]
    steamid: NotRequired[str]


class User(BaseModel):
    auth_key_id: Optional[str] = None

    steamid: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .webpropertyasset import WebpropertyAsset, WebpropertyAssetTypedDict
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ResponseEnvelopeWebpropertyAssetTypedDict(TypedDict):
    result: NotRequired[WebpropertyAssetTypedDict]


class ResponseEnvelopeWebpropertyAsset(BaseModel):
    result: Optional[WebpropertyAsset] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class UpnpServiceTypedDict(TypedDict):
    control_url: NotRequired[str]
    event_sub_url: NotRequired[str]
    scpd_url: NotRequired[str]
    service_id: NotRequired[str]
    service_type: NotRequired[str]


class UpnpService(BaseModel):
    control_url: Optional[str] = None

    event_sub_url: Optional[str] = None

    scpd_url: Optional[str] = None

    service_id: Optional[str] = None

    service_type: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .ipp_attribute import IppAttribute, IppAttributeTypedDict
from censys_platform.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class IppTypedDict(TypedDict):
    attribute_cups_version: NotRequired[str]
    r"""The CUPS version, if any, specified in the list of attributes returned in a get-printer-attributes response or CUPS-get-printers response. Generally in the form 'x.y.z'."""
    attribute_ipp_versions: NotRequired[Nullable[List[str]]]
    r"""Each IPP version, if any, specified in the list of attributes returned in a get-printer-attributes response or CUPS-get-printers response. Always in the form 'x.y'."""
    attribute_printer_uris: NotRequired[Nullable[List[str]]]
    r"""Each printer URI, if any, specified in the list of attributes returned in a get-printer-attributes response or CUPS-get-printers response. Uses ipp(s) or http(s) scheme, followed by a hostname or IP, and then the path to a particular printer."""
    attributes: NotRequired[Nullable[List[IppAttributeTypedDict]]]
    r"""All IPP attributes included in any contentful responses obtained. Each has a name, list of values (potentially only one), and a tag denoting how the value should be interpreted."""
    cups_version: NotRequired[str]
    r"""The CUPS version, if any, specified in the Server header of an IPP get-attributes response."""
    major_version: NotRequired[int]
    r"""Major component of IPP version listed in the Server header of a response to an IPP get-printer-attributes request."""
    minor_version: NotRequired[int]
    r"""Minor component of IPP version listed in the Server header of a response to an IPP get-printer-attributes request."""
    version_string: NotRequired[str]
    r"""The specific IPP version returned in response to an IPP get-printer-attributes request. Always in the form 'IPP/x.y'"""


class Ipp(BaseModel):
    attribute_cups_version: Optional[str] = None
    r"""The CUPS version, if any, specified in the list of attributes returned in a get-printer-attributes response or CUPS-get-printers response. Generally in the form 'x.y.z'."""

    attribute_ipp_versions: OptionalNullable[List[str]] = UNSET
    r"""Each IPP version, if any, specified in the list of attributes returned in a get-printer-attributes response or CUPS-get-printers response. Always in the form 'x.y'."""

    attribute_printer_uris: OptionalNullable[List[str]] = UNSET
    r"""Each printer URI, if any, specified in the list of attributes returned in a get-printer-attributes response or CUPS-get-printers response. Uses ipp(s) or http(s) scheme, followed by a hostname or IP, and then the path to a particular printer."""

    attributes: OptionalNullable[List[IppAttribute]] = UNSET
    r"""All IPP attributes included in any contentful responses obtained. Each has a name, list of values (potentially only one), and a tag denoting how the value should be interpreted."""

    cups_version: Optional[str] = None
    r"""The CUPS version, if any, specified in the Server header of an IPP get-attributes response."""

    major_version: Optional[int] = None
    r"""Major component of IPP version listed in the Server header of a response to an IPP get-printer-attributes request."""

    minor_version: Optional[int] = None
    r"""Minor component of IPP version listed in the Server header of a response to an IPP get-printer-attributes request."""

    version_string: Optional[str] = None
    r"""The specific IPP version returned in response to an IPP get-printer-attributes request. Always in the form 'IPP/x.y'"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "attribute_cups_version",
            "attribute_ipp_versions",
            "attribute_printer_uris",
            "attributes",
            "cups_version",
            "major_version",
            "minor_version",
            "version_string",
        ]
        nullable_fields = [
            "attribute_ipp_versions",
            "attribute_printer_uris",
            "attributes",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class AmqpVersionTypedDict(TypedDict):
    major: NotRequired[int]
    minor: NotRequired[int]
    revision: NotRequired[int]


class AmqpVersion(BaseModel):
    major: Optional[int] = None

    minor: Optional[int] = None

    revision: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CrudUpdateInputBodyTypedDict(TypedDict):
    name: str
    r"""name of the collection"""
    query: str
    r"""query string to search upon to build the collection"""
    description: NotRequired[str]
    r"""description of the collection"""


class CrudUpdateInputBody(BaseModel):
    name: str
    r"""name of the collection"""

    query: str
    r"""query string to search upon to build the collection"""

    description: Optional[str] = None
    r"""description of the collection"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class TacacsPlusTypedDict(TypedDict):
    data_length: NotRequired[int]
    flags: NotRequired[int]
    obfuscated: NotRequired[str]
    seq_num: NotRequired[int]
    session_id: NotRequired[int]
    type: NotRequired[int]
    version: NotRequired[int]


class TacacsPlus(BaseModel):
    data_length: Optional[int] = None

    flags: Optional[int] = None

    obfuscated: Optional[str] = None

    seq_num: Optional[int] = None

    session_id: Optional[int] = None

    type: Optional[int] = None

    version: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .ipmi_capabilities_authstatus import (
    IpmiCapabilitiesAuthStatus,
    IpmiCapabilitiesAuthStatusTypedDict,
)
from .ipmi_capabilities_completioncode import (
    IpmiCapabilitiesCompletionCode,
    IpmiCapabilitiesCompletionCodeTypedDict,
)
from .ipmi_capabilities_extendedcapabilities import (
    IpmiCapabilitiesExtendedCapabilities,
    IpmiCapabilitiesExtendedCapabilitiesTypedDict,
)
from .ipmi_capabilities_supportedauthtypes import (
    IpmiCapabilitiesSupportedAuthTypes,
    IpmiCapabilitiesSupportedAuthTypesTypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class IpmiCapabilitiesTypedDict(TypedDict):
    auth_status: NotRequired[IpmiCapabilitiesAuthStatusTypedDict]
    channel_number: NotRequired[int]
    r"""The response channel number"""
    completion_code: NotRequired[IpmiCapabilitiesCompletionCodeTypedDict]
    extended_capabilities: NotRequired[IpmiCapabilitiesExtendedCapabilitiesTypedDict]
    oem_data: NotRequired[int]
    r"""The OEM-specific data"""
    oem_id: NotRequired[str]
    r"""The 3-byte OEM identifier"""
    supported_auth_types: NotRequired[IpmiCapabilitiesSupportedAuthTypesTypedDict]


class IpmiCapabilities(BaseModel):
    auth_status: Optional[IpmiCapabilitiesAuthStatus] = None

    channel_number: Optional[int] = None
    r"""The response channel number"""

    completion_code: Optional[IpmiCapabilitiesCompletionCode] = None

    extended_capabilities: Optional[IpmiCapabilitiesExtendedCapabilities] = None

    oem_data: Optional[int] = None
    r"""The OEM-specific data"""

    oem_id: Optional[str] = None
    r"""The 3-byte OEM identifier"""

    supported_auth_types: Optional[IpmiCapabilitiesSupportedAuthTypes] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .mongodb_buildinfo_buildenvironment import (
    MongodbBuildInfoBuildEnvironment,
    MongodbBuildInfoBuildEnvironmentTypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MongodbBuildInfoTypedDict(TypedDict):
    build_environment: NotRequired[MongodbBuildInfoBuildEnvironmentTypedDict]
    git_version: NotRequired[str]
    r"""Version of mongodb server"""
    version: NotRequired[str]
    r"""Version of mongodb server"""


class MongodbBuildInfo(BaseModel):
    build_environment: Optional[MongodbBuildInfoBuildEnvironment] = None

    git_version: Optional[str] = None
    r"""Version of mongodb server"""

    version: Optional[str] = None
    r"""Version of mongodb server"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .ipmi_commandpayload_packednetfn_logicalunitnumber import (
    IpmiCommandPayloadPackedNetFnLogicalUnitNumber,
    IpmiCommandPayloadPackedNetFnLogicalUnitNumberTypedDict,
)
from .ipmi_commandpayload_packednetfn_netfn import (
    IpmiCommandPayloadPackedNetFnNetFn,
    IpmiCommandPayloadPackedNetFnNetFnTypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class IpmiCommandPayloadPackedNetFnTypedDict(TypedDict):
    logical_unit_number: NotRequired[
        IpmiCommandPayloadPackedNetFnLogicalUnitNumberTypedDict
    ]
    net_fn: NotRequired[IpmiCommandPayloadPackedNetFnNetFnTypedDict]
    raw: NotRequired[int]
    r"""The raw value of the (NetFn << 2) | LUN"""


class IpmiCommandPayloadPackedNetFn(BaseModel):
    logical_unit_number: Optional[IpmiCommandPayloadPackedNetFnLogicalUnitNumber] = None

    net_fn: Optional[IpmiCommandPayloadPackedNetFnNetFn] = None

    raw: Optional[int] = None
    r"""The raw value of the (NetFn << 2) | LUN"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from censys_platform.utils import FieldMetadata, PathParamMetadata, QueryParamMetadata
import httpx
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V3GlobaldataAssetCertificateRawGlobalsTypedDict(TypedDict):
    organization_id: NotRequired[str]


class V3GlobaldataAssetCertificateRawGlobals(BaseModel):
    organization_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class V3GlobaldataAssetCertificateRawRequestTypedDict(TypedDict):
    certificate_id: str
    r"""The SHA-256 certificate fingerprint."""
    organization_id: NotRequired[str]
    r"""The ID of a Censys organization to associate the request with. If omitted, the request will be processed using the authenticated user's free wallet where applicable. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information."""


class V3GlobaldataAssetCertificateRawRequest(BaseModel):
    certificate_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The SHA-256 certificate fingerprint."""

    organization_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""The ID of a Censys organization to associate the request with. If omitted, the request will be processed using the authenticated user's free wallet where applicable. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information."""


class V3GlobaldataAssetCertificateRawResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: httpx.Response


class V3GlobaldataAssetCertificateRawResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: httpx.Response



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class IpmiRMCPHeaderMessageClassTypedDict(TypedDict):
    class_: NotRequired[int]
    r"""Just the class part of the byte (lower 5 bits of raw)"""
    is_ack: NotRequired[bool]
    r"""True if the message is an acknowledgment to a previous message."""
    name: NotRequired[str]
    r"""The human-readable name of the message class"""
    raw: NotRequired[int]
    r"""The raw message class byte."""


class IpmiRMCPHeaderMessageClass(BaseModel):
    class_: Annotated[Optional[int], pydantic.Field(alias="class")] = None
    r"""Just the class part of the byte (lower 5 bits of raw)"""

    is_ack: Optional[bool] = None
    r"""True if the message is an acknowledgment to a previous message."""

    name: Optional[str] = None
    r"""The human-readable name of the message class"""

    raw: Optional[int] = None
    r"""The raw message class byte."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .searchqueryresponse import SearchQueryResponse, SearchQueryResponseTypedDict
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ResponseEnvelopeSearchQueryResponseTypedDict(TypedDict):
    result: NotRequired[SearchQueryResponseTypedDict]


class ResponseEnvelopeSearchQueryResponse(BaseModel):
    result: Optional[SearchQueryResponse] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .coordinates import Coordinates, CoordinatesTypedDict
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class LocationTypedDict(TypedDict):
    city: NotRequired[str]
    r"""The English name of the detected city."""
    continent: NotRequired[str]
    r"""The English name of the detected continent (North America, Europe, Asia, South America, Africa, Oceania, Antarctica)."""
    coordinates: NotRequired[CoordinatesTypedDict]
    country: NotRequired[str]
    r"""The English name of the detected country."""
    country_code: NotRequired[str]
    r"""The detected two-letter ISO 3166-1 alpha-2 country code (US, CN, GB, RU, ...)."""
    postal_code: NotRequired[str]
    r"""The postal code (if applicable) of the detected location."""
    province: NotRequired[str]
    r"""The state or province name of the detected location."""
    registered_country: NotRequired[str]
    r"""The English name of the registered country."""
    registered_country_code: NotRequired[str]
    r"""The registered country's two-letter ISO 3166-1 alpha-2 country code (US, CN, GB, RU, ...)."""
    timezone: NotRequired[str]
    r"""The IANA time zone database name of the detected location."""


class Location(BaseModel):
    city: Optional[str] = None
    r"""The English name of the detected city."""

    continent: Optional[str] = None
    r"""The English name of the detected continent (North America, Europe, Asia, South America, Africa, Oceania, Antarctica)."""

    coordinates: Optional[Coordinates] = None

    country: Optional[str] = None
    r"""The English name of the detected country."""

    country_code: Optional[str] = None
    r"""The detected two-letter ISO 3166-1 alpha-2 country code (US, CN, GB, RU, ...)."""

    postal_code: Optional[str] = None
    r"""The postal code (if applicable) of the detected location."""

    province: Optional[str] = None
    r"""The state or province name of the detected location."""

    registered_country: Optional[str] = None
    r"""The English name of the registered country."""

    registered_country_code: Optional[str] = None
    r"""The registered country's two-letter ISO 3166-1 alpha-2 country code (US, CN, GB, RU, ...)."""

    timezone: Optional[str] = None
    r"""The IANA time zone database name of the detected location."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SipTypedDict(TypedDict):
    code: NotRequired[int]
    server: NotRequired[str]
    r"""Server software reported by service"""
    status: NotRequired[str]
    version: NotRequired[str]
    r"""SIP version"""


class Sip(BaseModel):
    code: Optional[int] = None

    server: Optional[str] = None
    r"""Server software reported by service"""

    status: Optional[str] = None

    version: Optional[str] = None
    r"""SIP version"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing_extensions import TypedDict


class FieldValuePairTypedDict(TypedDict):
    field: str
    r"""The field to match"""
    value: str
    r"""The value to match"""


class FieldValuePair(BaseModel):
    field: str
    r"""The field to match"""

    value: str
    r"""The value to match"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .saprouter_routerinfo import SapRouterRouterInfo, SapRouterRouterInfoTypedDict
from .saprouter_routerversioninfo import (
    SapRouterRouterVersionInfo,
    SapRouterRouterVersionInfoTypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SapRouterTypedDict(TypedDict):
    router_info: NotRequired[SapRouterRouterInfoTypedDict]
    router_version_info: NotRequired[SapRouterRouterVersionInfoTypedDict]


class SapRouter(BaseModel):
    router_info: Optional[SapRouterRouterInfo] = None

    router_version_info: Optional[SapRouterRouterVersionInfo] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .autonomoussystem import AutonomousSystem, AutonomousSystemTypedDict
from .fielddiff import FieldDiff, FieldDiffTypedDict
from censys_platform.types import BaseModel
from typing import Dict, Optional
from typing_extensions import NotRequired, TypedDict


class RouteUpdatedTypedDict(TypedDict):
    diff: NotRequired[Dict[str, FieldDiffTypedDict]]
    route: NotRequired[AutonomousSystemTypedDict]


class RouteUpdated(BaseModel):
    diff: Optional[Dict[str, FieldDiff]] = None

    route: Optional[AutonomousSystem] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .pgbouncer_startupcapabilities import (
    PgbouncerStartupCapabilities,
    PgbouncerStartupCapabilitiesTypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PgbouncerTypedDict(TypedDict):
    startup_capabilities: NotRequired[PgbouncerStartupCapabilitiesTypedDict]


class Pgbouncer(BaseModel):
    startup_capabilities: Optional[PgbouncerStartupCapabilities] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class SocksTypedDict(TypedDict):
    no_authentication_required: NotRequired[bool]
    preferred_authentication: NotRequired[str]
    preferred_authentication_value: NotRequired[int]
    socks_version: NotRequired[int]


class Socks(BaseModel):
    no_authentication_required: Optional[bool] = None

    preferred_authentication: Optional[str] = None

    preferred_authentication_value: Optional[int] = None

    socks_version: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EdiPartyNameTypedDict(TypedDict):
    name_assigner: NotRequired[str]
    party_name: NotRequired[str]


class EdiPartyName(BaseModel):
    name_assigner: Optional[str] = None

    party_name: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .responseenvelopelistcollectionsresponsev1 import (
    ResponseEnvelopeListCollectionsResponseV1,
    ResponseEnvelopeListCollectionsResponseV1TypedDict,
)
from censys_platform.types import BaseModel
from censys_platform.utils import FieldMetadata, QueryParamMetadata
from typing import Dict, List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class V3CollectionsCrudListGlobalsTypedDict(TypedDict):
    organization_id: NotRequired[str]


class V3CollectionsCrudListGlobals(BaseModel):
    organization_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None


class V3CollectionsCrudListRequestTypedDict(TypedDict):
    organization_id: NotRequired[str]
    r"""The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information."""
    page_token: NotRequired[str]
    r"""page token for the requested page of collection results"""
    page_size: NotRequired[int]
    r"""amount of results to return per page"""


class V3CollectionsCrudListRequest(BaseModel):
    organization_id: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information."""

    page_token: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""page token for the requested page of collection results"""

    page_size: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""amount of results to return per page"""


class V3CollectionsCrudListResponseTypedDict(TypedDict):
    headers: Dict[str, List[str]]
    result: ResponseEnvelopeListCollectionsResponseV1TypedDict


class V3CollectionsCrudListResponse(BaseModel):
    headers: Dict[str, List[str]]

    result: ResponseEnvelopeListCollectionsResponseV1



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .authorityinfoaccess import AuthorityInfoAccess, AuthorityInfoAccessTypedDict
from .basicconstraints import BasicConstraints, BasicConstraintsTypedDict
from .cabforganizationid import CabfOrganizationID, CabfOrganizationIDTypedDict
from .certificatepolicy import CertificatePolicy, CertificatePolicyTypedDict
from .extendedkeyusage import ExtendedKeyUsage, ExtendedKeyUsageTypedDict
from .generalnames import GeneralNames, GeneralNamesTypedDict
from .keyusage import KeyUsage, KeyUsageTypedDict
from .nameconstraints import NameConstraints, NameConstraintsTypedDict
from .qcstatements import QcStatements, QcStatementsTypedDict
from .signedcertificatetimestamp import (
    SignedCertificateTimestamp,
    SignedCertificateTimestampTypedDict,
)
from .torservicedescriptor import TorServiceDescriptor, TorServiceDescriptorTypedDict
from censys_platform.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class CertificateExtensionsTypedDict(TypedDict):
    authority_info_access: NotRequired[AuthorityInfoAccessTypedDict]
    authority_key_id: NotRequired[str]
    r"""A key identifier, usually a digest of the DER-encoded SubjectPublicKeyInfo."""
    basic_constraints: NotRequired[BasicConstraintsTypedDict]
    cabf_organization_id: NotRequired[CabfOrganizationIDTypedDict]
    certificate_policies: NotRequired[Nullable[List[CertificatePolicyTypedDict]]]
    r"""The parsed id-ce-certificatePolicies extension (OID: 2.5.29.32)."""
    crl_distribution_points: NotRequired[Nullable[List[str]]]
    r"""The parsed id-ce-cRLDistributionPoints extension (OID: 2.5.29.31). Contents are a list of distributionPoint URLs; other distributionPoint types are omitted)."""
    ct_poison: NotRequired[bool]
    r"""Whether the certificate possesses the pre-certificate \"poison\" extension (OID: 1.3.6.1.4.1.11129.2.4.3)."""
    extended_key_usage: NotRequired[ExtendedKeyUsageTypedDict]
    issuer_alt_name: NotRequired[GeneralNamesTypedDict]
    key_usage: NotRequired[KeyUsageTypedDict]
    name_constraints: NotRequired[NameConstraintsTypedDict]
    qc_statements: NotRequired[QcStatementsTypedDict]
    signed_certificate_timestamps: NotRequired[
        Nullable[List[SignedCertificateTimestampTypedDict]]
    ]
    subject_alt_name: NotRequired[GeneralNamesTypedDict]
    subject_key_id: NotRequired[str]
    r"""A key identifier, usually a digest of the DER-encoded SubjectPublicKeyInfo.."""
    tor_service_descriptors: NotRequired[Nullable[List[TorServiceDescriptorTypedDict]]]


class CertificateExtensions(BaseModel):
    authority_info_access: Optional[AuthorityInfoAccess] = None

    authority_key_id: Optional[str] = None
    r"""A key identifier, usually a digest of the DER-encoded SubjectPublicKeyInfo."""

    basic_constraints: Optional[BasicConstraints] = None

    cabf_organization_id: Optional[CabfOrganizationID] = None

    certificate_policies: OptionalNullable[List[CertificatePolicy]] = UNSET
    r"""The parsed id-ce-certificatePolicies extension (OID: 2.5.29.32)."""

    crl_distribution_points: OptionalNullable[List[str]] = UNSET
    r"""The parsed id-ce-cRLDistributionPoints extension (OID: 2.5.29.31). Contents are a list of distributionPoint URLs; other distributionPoint types are omitted)."""

    ct_poison: Optional[bool] = None
    r"""Whether the certificate possesses the pre-certificate \"poison\" extension (OID: 1.3.6.1.4.1.11129.2.4.3)."""

    extended_key_usage: Optional[ExtendedKeyUsage] = None

    issuer_alt_name: Optional[GeneralNames] = None

    key_usage: Optional[KeyUsage] = None

    name_constraints: Optional[NameConstraints] = None

    qc_statements: Optional[QcStatements] = None

    signed_certificate_timestamps: OptionalNullable[
        List[SignedCertificateTimestamp]
    ] = UNSET

    subject_alt_name: Optional[GeneralNames] = None

    subject_key_id: Optional[str] = None
    r"""A key identifier, usually a digest of the DER-encoded SubjectPublicKeyInfo.."""

    tor_service_descriptors: OptionalNullable[List[TorServiceDescriptor]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "authority_info_access",
            "authority_key_id",
            "basic_constraints",
            "cabf_organization_id",
            "certificate_policies",
            "crl_distribution_points",
            "ct_poison",
            "extended_key_usage",
            "issuer_alt_name",
            "key_usage",
            "name_constraints",
            "qc_statements",
            "signed_certificate_timestamps",
            "subject_alt_name",
            "subject_key_id",
            "tor_service_descriptors",
        ]
        nullable_fields = [
            "certificate_policies",
            "crl_distribution_points",
            "signed_certificate_timestamps",
            "tor_service_descriptors",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .elasticsearch_systeminfo_ver import (
    ElasticSearchSystemInfoVer,
    ElasticSearchSystemInfoVerTypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ElasticSearchSystemInfoTypedDict(TypedDict):
    cluster_uuid: NotRequired[str]
    name: NotRequired[str]
    tagline: NotRequired[str]
    version: NotRequired[ElasticSearchSystemInfoVerTypedDict]


class ElasticSearchSystemInfo(BaseModel):
    cluster_uuid: Optional[str] = None

    name: Optional[str] = None

    tagline: Optional[str] = None

    version: Optional[ElasticSearchSystemInfoVer] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class RecordingCapabilitiesTypedDict(TypedDict):
    dynamic_recordings: NotRequired[bool]
    dynamic_tracks: NotRequired[bool]
    encoding: NotRequired[str]
    max_rate: NotRequired[int]
    max_recordings: NotRequired[int]
    max_recordings_job: NotRequired[int]
    max_total_rate: NotRequired[int]
    options: NotRequired[bool]


class RecordingCapabilities(BaseModel):
    dynamic_recordings: Optional[bool] = None

    dynamic_tracks: Optional[bool] = None

    encoding: Optional[str] = None

    max_rate: Optional[int] = None

    max_recordings: Optional[int] = None

    max_recordings_job: Optional[int] = None

    max_total_rate: Optional[int] = None

    options: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class KubernetesEndpointSubsetEndpointAddressTypedDict(TypedDict):
    hostname: NotRequired[str]
    ip: NotRequired[str]
    node_name: NotRequired[str]


class KubernetesEndpointSubsetEndpointAddress(BaseModel):
    hostname: Optional[str] = None

    ip: Optional[str] = None

    node_name: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class IpmiCommandPayloadPackedNetFnLogicalUnitNumberTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The human-readable name of the LUN"""
    raw: NotRequired[int]
    r"""The value of the LUN (3 bits)"""


class IpmiCommandPayloadPackedNetFnLogicalUnitNumber(BaseModel):
    name: Optional[str] = None
    r"""The human-readable name of the LUN"""

    raw: Optional[int] = None
    r"""The value of the LUN (3 bits)"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .mongodb_buildinfo import MongodbBuildInfo, MongodbBuildInfoTypedDict
from .mongodb_ismaster import MongodbIsMaster, MongodbIsMasterTypedDict
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MongodbTypedDict(TypedDict):
    build_info: NotRequired[MongodbBuildInfoTypedDict]
    is_master: NotRequired[MongodbIsMasterTypedDict]


class Mongodb(BaseModel):
    build_info: Optional[MongodbBuildInfo] = None

    is_master: Optional[MongodbIsMaster] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MediaStreamingTypedDict(TypedDict):
    non_aggregate_control: NotRequired[bool]
    rtp_multicast: NotRequired[bool]
    rtp_rtsp_tcp: NotRequired[bool]
    rtp_tcp: NotRequired[bool]


class MediaStreaming(BaseModel):
    non_aggregate_control: Optional[bool] = None

    rtp_multicast: Optional[bool] = None

    rtp_rtsp_tcp: Optional[bool] = None

    rtp_tcp: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class S7TypedDict(TypedDict):
    copyright: NotRequired[str]
    cpu_profile: NotRequired[str]
    firmware: NotRequired[str]
    hardware: NotRequired[str]
    location: NotRequired[str]
    memory_serial_number: NotRequired[str]
    module: NotRequired[str]
    module_id: NotRequired[str]
    module_type: NotRequired[str]
    oem_id: NotRequired[str]
    plant_id: NotRequired[str]
    reserved_for_os: NotRequired[str]
    serial_number: NotRequired[str]
    system: NotRequired[str]


class S7(BaseModel):
    copyright: Optional[str] = None

    cpu_profile: Optional[str] = None

    firmware: Optional[str] = None

    hardware: Optional[str] = None

    location: Optional[str] = None

    memory_serial_number: Optional[str] = None

    module: Optional[str] = None

    module_id: Optional[str] = None

    module_type: Optional[str] = None

    oem_id: Optional[str] = None

    plant_id: Optional[str] = None

    reserved_for_os: Optional[str] = None

    serial_number: Optional[str] = None

    system: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class OpenvpnTypedDict(TypedDict):
    accepts_v1: NotRequired[bool]
    accepts_v2: NotRequired[bool]


class Openvpn(BaseModel):
    accepts_v1: Optional[bool] = None

    accepts_v2: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class NTPNTPHeaderTypedDict(TypedDict):
    leap_indicator: NotRequired[int]
    r"""An enumerated value from 0 to 3 signifying whether a leap second will occur at the end of the current month. 0 signifies no leap second, 1 signifies an additive leap second, 2 signifies a subtractive leap second, and 3 signifies the state is unknown."""
    mode: NotRequired[int]
    r"""An enumerated value from 0 to 7 signifying the operational mode of the server."""
    poll: NotRequired[int]
    r"""The interval within which the server will expect a subsequent synchronization message, in log2 seconds."""
    precision: NotRequired[int]
    r"""The precision of the system's clock, in log2 seconds."""
    reference_id: NotRequired[str]
    r"""The identifier of the reference clock. For servers in stratum 1, one of an IANA-maintained list of sources. For servers in stratum 2, the ID of the stratum 1 server from which the time was retrieved (usually, its IP address), etc."""
    stratum: NotRequired[int]
    r"""The number of servers between a client and a non-NTP time source. 1 signifies that the server is authoritative, having direct access to a sensor. 2 signifies that the server got its time from a \"stratum 1\" server, etc. 16 means the clock is unsynchronized."""
    version: NotRequired[int]
    r"""The NTP version indicated in the server's response."""


class NTPNTPHeader(BaseModel):
    leap_indicator: Optional[int] = None
    r"""An enumerated value from 0 to 3 signifying whether a leap second will occur at the end of the current month. 0 signifies no leap second, 1 signifies an additive leap second, 2 signifies a subtractive leap second, and 3 signifies the state is unknown."""

    mode: Optional[int] = None
    r"""An enumerated value from 0 to 7 signifying the operational mode of the server."""

    poll: Optional[int] = None
    r"""The interval within which the server will expect a subsequent synchronization message, in log2 seconds."""

    precision: Optional[int] = None
    r"""The precision of the system's clock, in log2 seconds."""

    reference_id: Optional[str] = None
    r"""The identifier of the reference clock. For servers in stratum 1, one of an IANA-maintained list of sources. For servers in stratum 2, the ID of the stratum 1 server from which the time was retrieved (usually, its IP address), etc."""

    stratum: Optional[int] = None
    r"""The number of servers between a client and a non-NTP time source. 1 signifies that the server is authoritative, having direct access to a sensor. 2 signifies that the server got its time from a \"stratum 1\" server, etc. 16 means the clock is unsynchronized."""

    version: Optional[int] = None
    r"""The NTP version indicated in the server's response."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class DvrIPPartitionCapabilityTypedDict(TypedDict):
    max_partition_number: NotRequired[int]
    supported: NotRequired[bool]


class DvrIPPartitionCapability(BaseModel):
    max_partition_number: Optional[int] = None

    supported: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EvidenceTypedDict(TypedDict):
    data_path: NotRequired[str]
    exists: NotRequired[bool]
    found_value: NotRequired[str]
    literal_match: NotRequired[str]
    negative: NotRequired[bool]
    proprietary: NotRequired[bool]
    regex: NotRequired[str]
    semver_expression: NotRequired[str]


class Evidence(BaseModel):
    data_path: Optional[str] = None

    exists: Optional[bool] = None

    found_value: Optional[str] = None

    literal_match: Optional[str] = None

    negative: Optional[bool] = None

    proprietary: Optional[bool] = None

    regex: Optional[str] = None

    semver_expression: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from censys_platform import models, utils
from censys_platform._hooks import HookContext
from censys_platform.types import BaseModel, OptionalNullable, UNSET
from censys_platform.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union, cast


class ThreatHunting(BaseSDK):
    r"""Endpoints related to the Threat Hunting product"""

    def get_host_observations_with_certificate(
        self,
        *,
        request: Union[
            models.V3ThreathuntingGetHostObservationsWithCertificateRequest,
            models.V3ThreathuntingGetHostObservationsWithCertificateRequestTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3ThreathuntingGetHostObservationsWithCertificateResponse:
        r"""Get host history for a certificate

        Retrieve the historical observations of hosts associated with a certificate. This is useful for threat hunting, detection engineering, and timeline generation. Certificate history is also visible to Threat Hunting users in the Platform UI on the [certificate timeline](https://docs.censys.com/docs/platform-threat-hunting-use-cert-history-to-build-better-detections#/).<br><br>You can define a specific time frame of interest. If you do not specify a time frame, this endpoint will search the historical dataset that is available to your account. You may also filter results by port and transport protocol.<br><br>This endpoint is available to organizations that have access to the Threat Hunting module. It costs 5 credits per page of results.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, models.V3ThreathuntingGetHostObservationsWithCertificateRequest
            )
        request = cast(
            models.V3ThreathuntingGetHostObservationsWithCertificateRequest, request
        )

        req = self._build_request(
            method="GET",
            path="/v3/threat-hunting/certificate/{certificate_id}/observations/hosts",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.V3ThreathuntingGetHostObservationsWithCertificateGlobals(
                organization_id=self.sdk_configuration.globals.organization_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="v3-threathunting-get-host-observations-with-certificate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "403", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.V3ThreathuntingGetHostObservationsWithCertificateResponse(
                result=unmarshal_json_response(
                    models.ResponseEnvelopeHostObservationResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.AuthenticationErrorData, http_res
            )
            raise models.AuthenticationError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/problem+json"):
            response_data = unmarshal_json_response(models.ErrorModelData, http_res)
            raise models.ErrorModel(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def get_host_observations_with_certificate_async(
        self,
        *,
        request: Union[
            models.V3ThreathuntingGetHostObservationsWithCertificateRequest,
            models.V3ThreathuntingGetHostObservationsWithCertificateRequestTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3ThreathuntingGetHostObservationsWithCertificateResponse:
        r"""Get host history for a certificate

        Retrieve the historical observations of hosts associated with a certificate. This is useful for threat hunting, detection engineering, and timeline generation. Certificate history is also visible to Threat Hunting users in the Platform UI on the [certificate timeline](https://docs.censys.com/docs/platform-threat-hunting-use-cert-history-to-build-better-detections#/).<br><br>You can define a specific time frame of interest. If you do not specify a time frame, this endpoint will search the historical dataset that is available to your account. You may also filter results by port and transport protocol.<br><br>This endpoint is available to organizations that have access to the Threat Hunting module. It costs 5 credits per page of results.

        :param request: The request object to send.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        if not isinstance(request, BaseModel):
            request = utils.unmarshal(
                request, models.V3ThreathuntingGetHostObservationsWithCertificateRequest
            )
        request = cast(
            models.V3ThreathuntingGetHostObservationsWithCertificateRequest, request
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/threat-hunting/certificate/{certificate_id}/observations/hosts",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.V3ThreathuntingGetHostObservationsWithCertificateGlobals(
                organization_id=self.sdk_configuration.globals.organization_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="v3-threathunting-get-host-observations-with-certificate",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "403", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.V3ThreathuntingGetHostObservationsWithCertificateResponse(
                result=unmarshal_json_response(
                    models.ResponseEnvelopeHostObservationResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.AuthenticationErrorData, http_res
            )
            raise models.AuthenticationError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/problem+json"):
            response_data = unmarshal_json_response(models.ErrorModelData, http_res)
            raise models.ErrorModel(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def create_tracked_scan(
        self,
        *,
        scans_discovery_input_body: Union[
            models.ScansDiscoveryInputBody, models.ScansDiscoveryInputBodyTypedDict
        ],
        organization_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3ThreathuntingScansDiscoveryResponse:
        r"""Live Discovery: Initiate a new scan

        Initiate a scan to look for a currently unobserved service at a specific IP and port (`ip:port`) or hostname and port (`hostname:port`). This is equivalent to the [Live Discovery](https://docs.censys.com/docs/platform-threat-hunting-use-live-scan-and-rescan-to-validate-infrastructure#/) feature available in the UI, but you can also target web properties in addition to hosts.<br><br>The scan may take several minutes to complete. The response will contain a scan ID that you can use to [monitor the scan's status](https://docs.censys.com/reference/v3-threathunting-scans-get#/). After the scan completes, perform a lookup on the target asset to retrieve detailed scan information.<br><br>This endpoint is available to organizations that have access to the Threat Hunting module. It costs 15 credits to execute this endpoint.

        :param scans_discovery_input_body:
        :param organization_id: The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3ThreathuntingScansDiscoveryRequest(
            organization_id=organization_id,
            scans_discovery_input_body=utils.get_pydantic_model(
                scans_discovery_input_body, models.ScansDiscoveryInputBody
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v3/threat-hunting/scans/discovery",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.V3ThreathuntingScansDiscoveryGlobals(
                organization_id=self.sdk_configuration.globals.organization_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.scans_discovery_input_body,
                False,
                False,
                "json",
                models.ScansDiscoveryInputBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="v3-threathunting-scans-discovery",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "403", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.V3ThreathuntingScansDiscoveryResponse(
                result=unmarshal_json_response(
                    models.ResponseEnvelopeTrackedScan, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.AuthenticationErrorData, http_res
            )
            raise models.AuthenticationError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/problem+json"):
            response_data = unmarshal_json_response(models.ErrorModelData, http_res)
            raise models.ErrorModel(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def create_tracked_scan_async(
        self,
        *,
        scans_discovery_input_body: Union[
            models.ScansDiscoveryInputBody, models.ScansDiscoveryInputBodyTypedDict
        ],
        organization_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3ThreathuntingScansDiscoveryResponse:
        r"""Live Discovery: Initiate a new scan

        Initiate a scan to look for a currently unobserved service at a specific IP and port (`ip:port`) or hostname and port (`hostname:port`). This is equivalent to the [Live Discovery](https://docs.censys.com/docs/platform-threat-hunting-use-live-scan-and-rescan-to-validate-infrastructure#/) feature available in the UI, but you can also target web properties in addition to hosts.<br><br>The scan may take several minutes to complete. The response will contain a scan ID that you can use to [monitor the scan's status](https://docs.censys.com/reference/v3-threathunting-scans-get#/). After the scan completes, perform a lookup on the target asset to retrieve detailed scan information.<br><br>This endpoint is available to organizations that have access to the Threat Hunting module. It costs 15 credits to execute this endpoint.

        :param scans_discovery_input_body:
        :param organization_id: The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3ThreathuntingScansDiscoveryRequest(
            organization_id=organization_id,
            scans_discovery_input_body=utils.get_pydantic_model(
                scans_discovery_input_body, models.ScansDiscoveryInputBody
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v3/threat-hunting/scans/discovery",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.V3ThreathuntingScansDiscoveryGlobals(
                organization_id=self.sdk_configuration.globals.organization_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.scans_discovery_input_body,
                False,
                False,
                "json",
                models.ScansDiscoveryInputBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="v3-threathunting-scans-discovery",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "403", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.V3ThreathuntingScansDiscoveryResponse(
                result=unmarshal_json_response(
                    models.ResponseEnvelopeTrackedScan, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.AuthenticationErrorData, http_res
            )
            raise models.AuthenticationError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/problem+json"):
            response_data = unmarshal_json_response(models.ErrorModelData, http_res)
            raise models.ErrorModel(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def get_tracked_scan_threat_hunting(
        self,
        *,
        scan_id: str,
        organization_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3ThreathuntingScansGetResponse:
        r"""Get scan status

        Retrieve the current status of a scan by its ID. This endpoint works for both [Live Discovery scans](https://docs.censys.com/reference/v3-threathunting-scans-discovery#/) and [Live Rescans](https://docs.censys.com/reference/v3-globaldata-scans-rescan#/).<br><br>If the scan was successful, perform a lookup on the target asset to retrieve detailed scan information.<br><br>This endpoint is available to all Enterprise customers. This endpoint does not cost any credits to execute.

        :param scan_id: The unique identifier of the tracked scan
        :param organization_id: The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3ThreathuntingScansGetRequest(
            organization_id=organization_id,
            scan_id=scan_id,
        )

        req = self._build_request(
            method="GET",
            path="/v3/threat-hunting/scans/{scan_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/vnd.censys.api.v3.trackedscan.v1+json",
            http_headers=http_headers,
            _globals=models.V3ThreathuntingScansGetGlobals(
                organization_id=self.sdk_configuration.globals.organization_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="v3-threathunting-scans-get",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "403", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(
            http_res, "200", "application/vnd.censys.api.v3.trackedscan.v1+json"
        ):
            return models.V3ThreathuntingScansGetResponse(
                result=unmarshal_json_response(
                    models.ResponseEnvelopeTrackedScan, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.AuthenticationErrorData, http_res
            )
            raise models.AuthenticationError(response_data, http_res)
        if utils.match_response(http_res, ["403", "404"], "application/problem+json"):
            response_data = unmarshal_json_response(models.ErrorModelData, http_res)
            raise models.ErrorModel(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def get_tracked_scan_threat_hunting_async(
        self,
        *,
        scan_id: str,
        organization_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3ThreathuntingScansGetResponse:
        r"""Get scan status

        Retrieve the current status of a scan by its ID. This endpoint works for both [Live Discovery scans](https://docs.censys.com/reference/v3-threathunting-scans-discovery#/) and [Live Rescans](https://docs.censys.com/reference/v3-globaldata-scans-rescan#/).<br><br>If the scan was successful, perform a lookup on the target asset to retrieve detailed scan information.<br><br>This endpoint is available to all Enterprise customers. This endpoint does not cost any credits to execute.

        :param scan_id: The unique identifier of the tracked scan
        :param organization_id: The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3ThreathuntingScansGetRequest(
            organization_id=organization_id,
            scan_id=scan_id,
        )

        req = self._build_request_async(
            method="GET",
            path="/v3/threat-hunting/scans/{scan_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/vnd.censys.api.v3.trackedscan.v1+json",
            http_headers=http_headers,
            _globals=models.V3ThreathuntingScansGetGlobals(
                organization_id=self.sdk_configuration.globals.organization_id,
            ),
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="v3-threathunting-scans-get",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "403", "404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(
            http_res, "200", "application/vnd.censys.api.v3.trackedscan.v1+json"
        ):
            return models.V3ThreathuntingScansGetResponse(
                result=unmarshal_json_response(
                    models.ResponseEnvelopeTrackedScan, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.AuthenticationErrorData, http_res
            )
            raise models.AuthenticationError(response_data, http_res)
        if utils.match_response(http_res, ["403", "404"], "application/problem+json"):
            response_data = unmarshal_json_response(models.ErrorModelData, http_res)
            raise models.ErrorModel(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    def value_counts(
        self,
        *,
        search_value_counts_input_body: Union[
            models.SearchValueCountsInputBody,
            models.SearchValueCountsInputBodyTypedDict,
        ],
        organization_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3ThreathuntingValueCountsResponse:
        r"""CensEye: Retrieve value counts to discover pivots

        Get counts of web assets for specific field-value pairs and combinations of field-value pairs. This is similar to the [CensEye functionality](https://docs.censys.com/docs/platform-threat-hunting-use-censeye-to-build-detections#/) available in the Platform web UI, but it allows you to define specific fields of interest rather than the [default fields](https://docs.censys.com/docs/platform-threat-hunting-use-censeye-to-build-detections#default-pivot-fields) leveraged by the tool in the UI.<br><br>Each array can only target fields within the same nested object. For example, you can combine `host.services.port=80` and `host.services.protocol=SSH` in the same array, but you cannot combine `host.services.port=80` and `host.location.country=”United States”` in the same array. You can input multiple arrays of objects in each API call.<br><br>To use this endpoint, your organization must have access to the Threat Hunting Module. This endpoint costs 1 credit per count condition (array of objects) included in the API call.

        :param search_value_counts_input_body:
        :param organization_id: The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3ThreathuntingValueCountsRequest(
            organization_id=organization_id,
            search_value_counts_input_body=utils.get_pydantic_model(
                search_value_counts_input_body, models.SearchValueCountsInputBody
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v3/threat-hunting/value-counts",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.V3ThreathuntingValueCountsGlobals(
                organization_id=self.sdk_configuration.globals.organization_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.search_value_counts_input_body,
                False,
                False,
                "json",
                models.SearchValueCountsInputBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="v3-threathunting-value-counts",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "403", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.V3ThreathuntingValueCountsResponse(
                result=unmarshal_json_response(
                    models.ResponseEnvelopeValueCountsResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.AuthenticationErrorData, http_res
            )
            raise models.AuthenticationError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/problem+json"):
            response_data = unmarshal_json_response(models.ErrorModelData, http_res)
            raise models.ErrorModel(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)

    async def value_counts_async(
        self,
        *,
        search_value_counts_input_body: Union[
            models.SearchValueCountsInputBody,
            models.SearchValueCountsInputBodyTypedDict,
        ],
        organization_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.V3ThreathuntingValueCountsResponse:
        r"""CensEye: Retrieve value counts to discover pivots

        Get counts of web assets for specific field-value pairs and combinations of field-value pairs. This is similar to the [CensEye functionality](https://docs.censys.com/docs/platform-threat-hunting-use-censeye-to-build-detections#/) available in the Platform web UI, but it allows you to define specific fields of interest rather than the [default fields](https://docs.censys.com/docs/platform-threat-hunting-use-censeye-to-build-detections#default-pivot-fields) leveraged by the tool in the UI.<br><br>Each array can only target fields within the same nested object. For example, you can combine `host.services.port=80` and `host.services.protocol=SSH` in the same array, but you cannot combine `host.services.port=80` and `host.location.country=”United States”` in the same array. You can input multiple arrays of objects in each API call.<br><br>To use this endpoint, your organization must have access to the Threat Hunting Module. This endpoint costs 1 credit per count condition (array of objects) included in the API call.

        :param search_value_counts_input_body:
        :param organization_id: The ID of a Censys organization to associate the request with. See the [Getting Started docs](https://docs.censys.com/reference/get-started#step-3-find-and-use-your-organization-id-optional) for more information.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.V3ThreathuntingValueCountsRequest(
            organization_id=organization_id,
            search_value_counts_input_body=utils.get_pydantic_model(
                search_value_counts_input_body, models.SearchValueCountsInputBody
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v3/threat-hunting/value-counts",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            _globals=models.V3ThreathuntingValueCountsGlobals(
                organization_id=self.sdk_configuration.globals.organization_id,
            ),
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.search_value_counts_input_body,
                False,
                False,
                "json",
                models.SearchValueCountsInputBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="v3-threathunting-value-counts",
                oauth2_scopes=None,
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["401", "403", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return models.V3ThreathuntingValueCountsResponse(
                result=unmarshal_json_response(
                    models.ResponseEnvelopeValueCountsResponse, http_res
                ),
                headers=utils.get_response_headers(http_res.headers),
            )
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                models.AuthenticationErrorData, http_res
            )
            raise models.AuthenticationError(response_data, http_res)
        if utils.match_response(http_res, "403", "application/problem+json"):
            response_data = unmarshal_json_response(models.ErrorModelData, http_res)
            raise models.ErrorModel(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.SDKError("API error occurred", http_res, http_res_text)

        raise models.SDKError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class OtherNameTypedDict(TypedDict):
    id: NotRequired[str]
    r"""The OID identifying the syntax of the otherName value."""
    value: NotRequired[str]
    r"""The raw otherName value."""


class OtherName(BaseModel):
    id: Optional[str] = None
    r"""The OID identifying the syntax of the otherName value."""

    value: Optional[str] = None
    r"""The raw otherName value."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MurmurMurmurVersionTypedDict(TypedDict):
    build: NotRequired[int]
    major: NotRequired[int]
    minor: NotRequired[int]
    os: NotRequired[str]
    os_version: NotRequired[str]
    version_string: NotRequired[str]


class MurmurMurmurVersion(BaseModel):
    build: Optional[int] = None

    major: Optional[int] = None

    minor: Optional[int] = None

    os: Optional[str] = None

    os_version: Optional[str] = None

    version_string: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .rdp_domainparameters import RdpDomainParameters, RdpDomainParametersTypedDict
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class RdpConnectResponseTypedDict(TypedDict):
    connect_id: NotRequired[int]
    domain_parameters: NotRequired[RdpDomainParametersTypedDict]


class RdpConnectResponse(BaseModel):
    connect_id: Optional[int] = None

    domain_parameters: Optional[RdpDomainParameters] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PrometheusResponsePrometheusVersionTypedDict(TypedDict):
    go_version: NotRequired[str]
    r"""Version of Go used to build Prometheus."""
    revision: NotRequired[str]
    r"""Revision of Prometheus."""
    version: NotRequired[str]
    r"""Version of Prometheus."""


class PrometheusResponsePrometheusVersion(BaseModel):
    go_version: Optional[str] = None
    r"""Version of Go used to build Prometheus."""

    revision: Optional[str] = None
    r"""Revision of Prometheus."""

    version: Optional[str] = None
    r"""Version of Prometheus."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .attribute import Attribute, AttributeTypedDict
from .certificate import Certificate, CertificateTypedDict
from .endpointscanstate import EndpointScanState, EndpointScanStateTypedDict
from .jarmscan import JarmScan, JarmScanTypedDict
from .label import Label, LabelTypedDict
from .risk import Risk, RiskTypedDict
from .threat import Threat, ThreatTypedDict
from .tls import TLS, TLSTypedDict
from .vuln import Vuln, VulnTypedDict
from censys_platform.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class WebpropertyTypedDict(TypedDict):
    cert: NotRequired[CertificateTypedDict]
    endpoints: NotRequired[Nullable[List[EndpointScanStateTypedDict]]]
    exposures: NotRequired[Nullable[List[RiskTypedDict]]]
    hardware: NotRequired[Nullable[List[AttributeTypedDict]]]
    hostname: NotRequired[str]
    jarm: NotRequired[JarmScanTypedDict]
    labels: NotRequired[Nullable[List[LabelTypedDict]]]
    misconfigs: NotRequired[Nullable[List[RiskTypedDict]]]
    operating_systems: NotRequired[Nullable[List[AttributeTypedDict]]]
    port: NotRequired[int]
    scan_time: NotRequired[str]
    software: NotRequired[Nullable[List[AttributeTypedDict]]]
    threats: NotRequired[Nullable[List[ThreatTypedDict]]]
    tls: NotRequired[TLSTypedDict]
    vulns: NotRequired[Nullable[List[VulnTypedDict]]]


class Webproperty(BaseModel):
    cert: Optional[Certificate] = None

    endpoints: OptionalNullable[List[EndpointScanState]] = UNSET

    exposures: OptionalNullable[List[Risk]] = UNSET

    hardware: OptionalNullable[List[Attribute]] = UNSET

    hostname: Optional[str] = None

    jarm: Optional[JarmScan] = None

    labels: OptionalNullable[List[Label]] = UNSET

    misconfigs: OptionalNullable[List[Risk]] = UNSET

    operating_systems: OptionalNullable[List[Attribute]] = UNSET

    port: Optional[int] = None

    scan_time: Optional[str] = None

    software: OptionalNullable[List[Attribute]] = UNSET

    threats: OptionalNullable[List[Threat]] = UNSET

    tls: Optional[TLS] = None

    vulns: OptionalNullable[List[Vuln]] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "cert",
            "endpoints",
            "exposures",
            "hardware",
            "hostname",
            "jarm",
            "labels",
            "misconfigs",
            "operating_systems",
            "port",
            "scan_time",
            "software",
            "threats",
            "tls",
            "vulns",
        ]
        nullable_fields = [
            "endpoints",
            "exposures",
            "hardware",
            "labels",
            "misconfigs",
            "operating_systems",
            "software",
            "threats",
            "vulns",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .alternativequery import AlternativeQuery, AlternativeQueryTypedDict
from .convertqueryerror import ConvertQueryError, ConvertQueryErrorTypedDict
from censys_platform.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class SearchConvertQueryResponseTargets(str, Enum):
    HOST = "host"
    WEB = "web"
    CERT = "cert"
    UNKNOWN = "unknown"


class SearchConvertQueryResponseTypedDict(TypedDict):
    errors: Nullable[List[ConvertQueryErrorTypedDict]]
    r"""The error messages if the query could not be converted."""
    original_query: str
    r"""The original Censys Search Language query."""
    targets: Nullable[List[SearchConvertQueryResponseTargets]]
    r"""The types of Platform data records (host, cert, and web) that can be targeted by the converted query. Note: Does not include targets that could be returned due to full-document search. This may be empty if the query is a match-all for full-document search query."""
    alternative_queries: NotRequired[Nullable[List[AlternativeQueryTypedDict]]]
    r"""Alternative queries that may be used. For example, a query that includes web results in addition to host results."""
    comments: NotRequired[Nullable[List[str]]]
    r"""Comments about the query conversion."""
    converted_query: NotRequired[str]
    r"""The original query, converted to CenQL syntax. This may be empty if unable to translate the query."""


class SearchConvertQueryResponse(BaseModel):
    errors: Nullable[List[ConvertQueryError]]
    r"""The error messages if the query could not be converted."""

    original_query: str
    r"""The original Censys Search Language query."""

    targets: Nullable[List[SearchConvertQueryResponseTargets]]
    r"""The types of Platform data records (host, cert, and web) that can be targeted by the converted query. Note: Does not include targets that could be returned due to full-document search. This may be empty if the query is a match-all for full-document search query."""

    alternative_queries: OptionalNullable[List[AlternativeQuery]] = UNSET
    r"""Alternative queries that may be used. For example, a query that includes web results in addition to host results."""

    comments: OptionalNullable[List[str]] = UNSET
    r"""Comments about the query conversion."""

    converted_query: Optional[str] = None
    r"""The original query, converted to CenQL syntax. This may be empty if unable to translate the query."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["alternative_queries", "comments", "converted_query"]
        nullable_fields = ["alternative_queries", "comments", "errors", "targets"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class FoxTypedDict(TypedDict):
    app_name: NotRequired[str]
    app_version: NotRequired[str]
    auth_agent_type: NotRequired[str]
    brand_id: NotRequired[str]
    host_address: NotRequired[str]
    host_id: NotRequired[str]
    hostname: NotRequired[str]
    id: NotRequired[int]
    language: NotRequired[str]
    os_name: NotRequired[str]
    os_version: NotRequired[str]
    station_name: NotRequired[str]
    sys_info: NotRequired[str]
    time_zone: NotRequired[str]
    version: NotRequired[str]
    vm_name: NotRequired[str]
    vm_uuid: NotRequired[str]
    vm_version: NotRequired[str]


class Fox(BaseModel):
    app_name: Optional[str] = None

    app_version: Optional[str] = None

    auth_agent_type: Optional[str] = None

    brand_id: Optional[str] = None

    host_address: Optional[str] = None

    host_id: Optional[str] = None

    hostname: Optional[str] = None

    id: Optional[int] = None

    language: Optional[str] = None

    os_name: Optional[str] = None

    os_version: Optional[str] = None

    station_name: Optional[str] = None

    sys_info: Optional[str] = None

    time_zone: Optional[str] = None

    version: Optional[str] = None

    vm_name: Optional[str] = None

    vm_uuid: Optional[str] = None

    vm_version: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .ciscoipsla_handshake import CiscoIpslaHandshake, CiscoIpslaHandshakeTypedDict
from .ciscoipsla_measureresponse import (
    CiscoIpslaMeasureResponse,
    CiscoIpslaMeasureResponseTypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CiscoIpslaTypedDict(TypedDict):
    handshake: NotRequired[CiscoIpslaHandshakeTypedDict]
    measure_response: NotRequired[CiscoIpslaMeasureResponseTypedDict]


class CiscoIpsla(BaseModel):
    handshake: Optional[CiscoIpslaHandshake] = None

    measure_response: Optional[CiscoIpslaMeasureResponse] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ModbusExceptionResponseTypedDict(TypedDict):
    exception_function: NotRequired[int]
    exception_type: NotRequired[int]


class ModbusExceptionResponse(BaseModel):
    exception_function: Optional[int] = None

    exception_type: Optional[int] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PcAnywhereStatusTypedDict(TypedDict):
    in_use: NotRequired[bool]
    r"""Workstation is In Use if true, Available if false"""
    raw: NotRequired[str]
    r"""Full 'ST' query response"""


class PcAnywhereStatus(BaseModel):
    in_use: Optional[bool] = None
    r"""Workstation is In Use if true, Available if false"""

    raw: Optional[str] = None
    r"""Full 'ST' query response"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class RdpSecurityProtocolTypedDict(TypedDict):
    credssp: NotRequired[bool]
    credssp_early_auth: NotRequired[bool]
    error: NotRequired[bool]
    error_bad_flags: NotRequired[bool]
    error_hybrid_required: NotRequired[bool]
    error_ssl_cert_missing: NotRequired[bool]
    error_ssl_forbidden: NotRequired[bool]
    error_ssl_required: NotRequired[bool]
    error_ssl_user_auth_required: NotRequired[bool]
    error_unknown: NotRequired[bool]
    raw_value: NotRequired[int]
    rdstls: NotRequired[bool]
    standard_rdp: NotRequired[bool]
    tls: NotRequired[bool]


class RdpSecurityProtocol(BaseModel):
    credssp: Optional[bool] = None

    credssp_early_auth: Optional[bool] = None

    error: Optional[bool] = None

    error_bad_flags: Optional[bool] = None

    error_hybrid_required: Optional[bool] = None

    error_ssl_cert_missing: Optional[bool] = None

    error_ssl_forbidden: Optional[bool] = None

    error_ssl_required: Optional[bool] = None

    error_ssl_user_auth_required: Optional[bool] = None

    error_unknown: Optional[bool] = None

    raw_value: Optional[int] = None

    rdstls: Optional[bool] = None

    standard_rdp: Optional[bool] = None

    tls: Optional[bool] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class TibiaTypedDict(TypedDict):
    client_version: NotRequired[str]
    location: NotRequired[str]
    login_ip: NotRequired[str]
    login_port: NotRequired[str]
    name: NotRequired[str]
    server: NotRequired[str]
    url: NotRequired[str]
    version: NotRequired[str]


class Tibia(BaseModel):
    client_version: Optional[str] = None

    location: Optional[str] = None

    login_ip: Optional[str] = None

    login_port: Optional[str] = None

    name: Optional[str] = None

    server: Optional[str] = None

    url: Optional[str] = None

    version: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class RocketmqResponseHeaderTypedDict(TypedDict):
    code: NotRequired[int]
    flag: NotRequired[int]
    language: NotRequired[str]
    opaque: NotRequired[int]
    serialize_type_current_rpc: NotRequired[str]


class RocketmqResponseHeader(BaseModel):
    code: Optional[int] = None

    flag: Optional[int] = None

    language: Optional[str] = None

    opaque: Optional[int] = None

    serialize_type_current_rpc: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class CabfOrganizationIDTypedDict(TypedDict):
    country: NotRequired[str]
    reference: NotRequired[str]
    scheme: NotRequired[str]
    state: NotRequired[str]


class CabfOrganizationID(BaseModel):
    country: Optional[str] = None

    reference: Optional[str] = None

    scheme: Optional[str] = None

    state: Optional[str] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .monerop2p_payload import MoneroP2PPayload, MoneroP2PPayloadTypedDict
from .monerop2p_responseheader import (
    MoneroP2PResponseHeader,
    MoneroP2PResponseHeaderTypedDict,
)
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class MoneroP2PPingResponseTypedDict(TypedDict):
    payload: NotRequired[MoneroP2PPayloadTypedDict]
    response_header: NotRequired[MoneroP2PResponseHeaderTypedDict]


class MoneroP2PPingResponse(BaseModel):
    payload: Optional[MoneroP2PPayload] = None

    response_header: Optional[MoneroP2PResponseHeader] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .activemq import Activemq, ActivemqTypedDict
from .amqp import Amqp, AmqpTypedDict
from .anyconnect import AnyConnect, AnyConnectTypedDict
from .asteriskmanagerinterface import (
    AsteriskManagerInterface,
    AsteriskManagerInterfaceTypedDict,
)
from .attribute import Attribute, AttributeTypedDict
from .bacnet import Bacnet, BacnetTypedDict
from .certificate import Certificate, CertificateTypedDict
from .checkpointtopology import CheckpointTopology, CheckpointTopologyTypedDict
from .chromecast import Chromecast, ChromecastTypedDict
from .ciscoipsla import CiscoIpsla, CiscoIpslaTypedDict
from .cmore import Cmore, CmoreTypedDict
from .coap import Coap, CoapTypedDict
from .crestroncp3 import CrestronCp3, CrestronCp3TypedDict
from .crestrondinap2 import CrestronDinAp2, CrestronDinAp2TypedDict
from .cwmp import Cwmp, CwmpTypedDict
from .darkcomet import Darkcomet, DarkcometTypedDict
from .darkgate import Darkgate, DarkgateTypedDict
from .dcerpc import Dcerpc, DcerpcTypedDict
from .dhcpdiscover import Dhcpdiscover, DhcpdiscoverTypedDict
from .dnp3 import Dnp3, Dnp3TypedDict
from .dns import DNS, DNSTypedDict
from .dtls import Dtls, DtlsTypedDict
from .dvrip import DvrIP, DvrIPTypedDict
from .eip import Eip, EipTypedDict
from .elffile import ElfFile, ElfFileTypedDict
from .endpointscanstate import EndpointScanState, EndpointScanStateTypedDict
from .epmd import Epmd, EpmdTypedDict
from .etcd import Etcd, EtcdTypedDict
from .ethereum import Ethereum, EthereumTypedDict
from .fox import Fox, FoxTypedDict
from .ftp import Ftp, FtpTypedDict
from .gearman import Gearman, GearmanTypedDict
from .hidvertx import HidVertx, HidVertxTypedDict
from .hikvision import Hikvision, HikvisionTypedDict
from .ibmnje import Ibmnje, IbmnjeTypedDict
from .ike import Ike, IkeTypedDict
from .imap import Imap, ImapTypedDict
from .iota import Iota, IotaTypedDict
from .ipmi import Ipmi, IpmiTypedDict
from .ipp import Ipp, IppTypedDict
from .iscsi import Iscsi, IscsiTypedDict
from .ja4tscanscan import Ja4TScanScan, Ja4TScanScanTypedDict
from .jarmscan import JarmScan, JarmScanTypedDict
from .krpc import Krpc, KrpcTypedDict
from .l2tp import L2Tp, L2TpTypedDict
from .label import Label, LabelTypedDict
from .ldap import Ldap, LdapTypedDict
from .lpd import Lpd, LpdTypedDict
from .mdns import Mdns, MdnsTypedDict
from .memcached import Memcached, MemcachedTypedDict
from .mikrotikwinbox import MikrotikWinbox, MikrotikWinboxTypedDict
from .minecraft import Minecraft, MinecraftTypedDict
from .mms import Mms, MmsTypedDict
from .modbus import Modbus, ModbusTypedDict
from .monerop2p import MoneroP2P, MoneroP2PTypedDict
from .mongodb import Mongodb, MongodbTypedDict
from .mqtt import Mqtt, MqttTypedDict
from .mssql import Mssql, MssqlTypedDict
from .murmur import Murmur, MurmurTypedDict
from .mysql import Mysql, MysqlTypedDict
from .natsio import NatsIo, NatsIoTypedDict
from .nbd import Nbd, NbdTypedDict
from .nfsmountd import NfsMountd, NfsMountdTypedDict
from .nmea import Nmea, NmeaTypedDict
from .ntp import Ntp, NtpTypedDict
from .ntrip import Ntrip, NtripTypedDict
from .onc import Onc, OncTypedDict
from .onvif import Onvif, OnvifTypedDict
from .opcua import OpcUa, OpcUaTypedDict
from .openvpn import Openvpn, OpenvpnTypedDict
from .openvpnmgmt import OpenvpnMgmt, OpenvpnMgmtTypedDict
from .oracle import Oracle, OracleTypedDict
from .pcanywhere import PcAnywhere, PcAnywhereTypedDict
from .pgbouncer import Pgbouncer, PgbouncerTypedDict
from .pop3 import Pop3, Pop3TypedDict
from .portmap import Portmap, PortmapTypedDict
from .postgres import Postgres, PostgresTypedDict
from .pptp import Pptp, PptpTypedDict
from .profinetcm import ProfinetCm, ProfinetCmTypedDict
from .rdate import Rdate, RdateTypedDict
from .rdp import Rdp, RdpTypedDict
from .realport import Realport, RealportTypedDict
from .redis import Redis, RedisTypedDict
from .redline import Redline, RedlineTypedDict
from .redlioncrimson import RedlionCrimson, RedlionCrimsonTypedDict
from .representativeinfo import RepresentativeInfo, RepresentativeInfoTypedDict
from .rifatron import Rifatron, RifatronTypedDict
from .ripple import Ripple, RippleTypedDict
from .risk import Risk, RiskTypedDict
from .rlogin import Rlogin, RloginTypedDict
from .rocketmq import Rocketmq, RocketmqTypedDict
from .rtsp import Rtsp, RtspTypedDict
from .s7 import S7, S7TypedDict
from .saprouter import SapRouter, SapRouterTypedDict
from .scpi import Scpi, ScpiTypedDict
from .screenshot import Screenshot, ScreenshotTypedDict
from .ser2net import Ser2Net, Ser2NetTypedDict
from .sevendaystodie import SevenDaysToDie, SevenDaysToDieTypedDict
from .sip import Sip, SipTypedDict
from .skinny import Skinny, SkinnyTypedDict
from .smb import Smb, SmbTypedDict
from .smtp import SMTP, SMTPTypedDict
from .snmp import Snmp, SnmpTypedDict
from .socks import Socks, SocksTypedDict
from .spice import Spice, SpiceTypedDict
from .ssdp import Ssdp, SsdpTypedDict
from .ssh import SSH, SSHTypedDict
from .steam import Steam, SteamTypedDict
from .tacacsplus import TacacsPlus, TacacsPlusTypedDict
from .teamviewer import TeamViewer, TeamViewerTypedDict
from .telnet import Telnet, TelnetTypedDict
from .threat import Threat, ThreatTypedDict
from .tibia import Tibia, TibiaTypedDict
from .tls import TLS, TLSTypedDict
from .tplinkkasa import TplinkKasa, TplinkKasaTypedDict
from .unitronicspcom import UnitronicsPcom, UnitronicsPcomTypedDict
from .upnp import Upnp, UpnpTypedDict
from .ventrilo import Ventrilo, VentriloTypedDict
from .vnc import Vnc, VncTypedDict
from .vuln import Vuln, VulnTypedDict
from .weblogict3 import WeblogicT3, WeblogicT3TypedDict
from .winrm import Winrm, WinrmTypedDict
from .wsdiscovery import WsDiscovery, WsDiscoveryTypedDict
from .x11 import X11, X11TypedDict
from .zeromq import Zeromq, ZeromqTypedDict
from censys_platform.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from enum import Enum
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class ServiceTransportProtocol(str, Enum):
    UNKNOWN = ""
    TCP = "tcp"
    UDP = "udp"
    ICMP = "icmp"
    QUIC = "quic"


class ServiceTypedDict(TypedDict):
    activemq: NotRequired[ActivemqTypedDict]
    amqp: NotRequired[AmqpTypedDict]
    any_connect: NotRequired[AnyConnectTypedDict]
    asterisk_manager_interface: NotRequired[AsteriskManagerInterfaceTypedDict]
    bacnet: NotRequired[BacnetTypedDict]
    banner: NotRequired[str]
    banner_hash_sha256: NotRequired[str]
    banner_hex: NotRequired[str]
    cert: NotRequired[CertificateTypedDict]
    checkpoint_topology: NotRequired[CheckpointTopologyTypedDict]
    chromecast: NotRequired[ChromecastTypedDict]
    cisco_ipsla: NotRequired[CiscoIpslaTypedDict]
    cmore: NotRequired[CmoreTypedDict]
    coap: NotRequired[CoapTypedDict]
    crestron_cp3: NotRequired[CrestronCp3TypedDict]
    crestron_din_ap2: NotRequired[CrestronDinAp2TypedDict]
    cwmp: NotRequired[CwmpTypedDict]
    darkcomet: NotRequired[DarkcometTypedDict]
    darkgate: NotRequired[DarkgateTypedDict]
    dcerpc: NotRequired[DcerpcTypedDict]
    dhcpdiscover: NotRequired[DhcpdiscoverTypedDict]
    dnp3: NotRequired[Dnp3TypedDict]
    dns: NotRequired[DNSTypedDict]
    dtls: NotRequired[DtlsTypedDict]
    dvr_ip: NotRequired[DvrIPTypedDict]
    eip: NotRequired[EipTypedDict]
    elf_file: NotRequired[ElfFileTypedDict]
    endpoints: NotRequired[Nullable[List[EndpointScanStateTypedDict]]]
    epmd: NotRequired[EpmdTypedDict]
    etcd: NotRequired[EtcdTypedDict]
    ethereum: NotRequired[EthereumTypedDict]
    exposures: NotRequired[Nullable[List[RiskTypedDict]]]
    fox: NotRequired[FoxTypedDict]
    ftp: NotRequired[FtpTypedDict]
    gearman: NotRequired[GearmanTypedDict]
    hardware: NotRequired[Nullable[List[AttributeTypedDict]]]
    hid_vertx: NotRequired[HidVertxTypedDict]
    hikvision: NotRequired[HikvisionTypedDict]
    ibmnje: NotRequired[IbmnjeTypedDict]
    ike: NotRequired[IkeTypedDict]
    imap: NotRequired[ImapTypedDict]
    iota: NotRequired[IotaTypedDict]
    ip: NotRequired[str]
    ipmi: NotRequired[IpmiTypedDict]
    ipp: NotRequired[IppTypedDict]
    iscsi: NotRequired[IscsiTypedDict]
    ja4tscan: NotRequired[Ja4TScanScanTypedDict]
    jarm: NotRequired[JarmScanTypedDict]
    krpc: NotRequired[KrpcTypedDict]
    l2tp: NotRequired[L2TpTypedDict]
    labels: NotRequired[Nullable[List[LabelTypedDict]]]
    ldap: NotRequired[LdapTypedDict]
    lpd: NotRequired[LpdTypedDict]
    mdns: NotRequired[MdnsTypedDict]
    memcached: NotRequired[MemcachedTypedDict]
    mikrotik_winbox: NotRequired[MikrotikWinboxTypedDict]
    minecraft: NotRequired[MinecraftTypedDict]
    misconfigs: NotRequired[Nullable[List[RiskTypedDict]]]
    mms: NotRequired[MmsTypedDict]
    modbus: NotRequired[ModbusTypedDict]
    monero_p2p: NotRequired[MoneroP2PTypedDict]
    mongodb: NotRequired[MongodbTypedDict]
    mqtt: NotRequired[MqttTypedDict]
    mssql: NotRequired[MssqlTypedDict]
    murmur: NotRequired[MurmurTypedDict]
    mysql: NotRequired[MysqlTypedDict]
    nats_io: NotRequired[NatsIoTypedDict]
    nbd: NotRequired[NbdTypedDict]
    nfs_mountd: NotRequired[NfsMountdTypedDict]
    nmea: NotRequired[NmeaTypedDict]
    ntp: NotRequired[NtpTypedDict]
    ntrip: NotRequired[NtripTypedDict]
    onc: NotRequired[OncTypedDict]
    onvif: NotRequired[OnvifTypedDict]
    opc_ua: NotRequired[OpcUaTypedDict]
    openvpn: NotRequired[OpenvpnTypedDict]
    openvpn_mgmt: NotRequired[OpenvpnMgmtTypedDict]
    operating_systems: NotRequired[Nullable[List[AttributeTypedDict]]]
    oracle: NotRequired[OracleTypedDict]
    pc_anywhere: NotRequired[PcAnywhereTypedDict]
    pgbouncer: NotRequired[PgbouncerTypedDict]
    pop3: NotRequired[Pop3TypedDict]
    port: NotRequired[int]
    portmap: NotRequired[PortmapTypedDict]
    postgres: NotRequired[PostgresTypedDict]
    pptp: NotRequired[PptpTypedDict]
    profinet_cm: NotRequired[ProfinetCmTypedDict]
    protocol: NotRequired[str]
    rdate: NotRequired[RdateTypedDict]
    rdp: NotRequired[RdpTypedDict]
    realport: NotRequired[RealportTypedDict]
    redis: NotRequired[RedisTypedDict]
    redline: NotRequired[RedlineTypedDict]
    redlion_crimson: NotRequired[RedlionCrimsonTypedDict]
    representative_info: NotRequired[RepresentativeInfoTypedDict]
    rifatron: NotRequired[RifatronTypedDict]
    ripple: NotRequired[RippleTypedDict]
    rlogin: NotRequired[RloginTypedDict]
    rocketmq: NotRequired[RocketmqTypedDict]
    rtsp: NotRequired[RtspTypedDict]
    s7: NotRequired[S7TypedDict]
    sap_router: NotRequired[SapRouterTypedDict]
    scan_time: NotRequired[str]
    scpi: NotRequired[ScpiTypedDict]
    screenshots: NotRequired[Nullable[List[ScreenshotTypedDict]]]
    ser2net: NotRequired[Ser2NetTypedDict]
    seven_days_to_die: NotRequired[SevenDaysToDieTypedDict]
    sip: NotRequired[SipTypedDict]
    skinny: NotRequired[SkinnyTypedDict]
    smb: NotRequired[SmbTypedDict]
    smtp: NotRequired[SMTPTypedDict]
    snmp: NotRequired[SnmpTypedDict]
    socks: NotRequired[SocksTypedDict]
    software: NotRequired[Nullable[List[AttributeTypedDict]]]
    spice: NotRequired[SpiceTypedDict]
    ssdp: NotRequired[SsdpTypedDict]
    ssh: NotRequired[SSHTypedDict]
    steam: NotRequired[SteamTypedDict]
    tacacs_plus: NotRequired[TacacsPlusTypedDict]
    team_viewer: NotRequired[TeamViewerTypedDict]
    telnet: NotRequired[TelnetTypedDict]
    threats: NotRequired[Nullable[List[ThreatTypedDict]]]
    tibia: NotRequired[TibiaTypedDict]
    tls: NotRequired[TLSTypedDict]
    tplink_kasa: NotRequired[TplinkKasaTypedDict]
    transport_protocol: NotRequired[ServiceTransportProtocol]
    unitronics_pcom: NotRequired[UnitronicsPcomTypedDict]
    upnp: NotRequired[UpnpTypedDict]
    ventrilo: NotRequired[VentriloTypedDict]
    vnc: NotRequired[VncTypedDict]
    vulns: NotRequired[Nullable[List[VulnTypedDict]]]
    weblogic_t3: NotRequired[WeblogicT3TypedDict]
    winrm: NotRequired[WinrmTypedDict]
    ws_discovery: NotRequired[WsDiscoveryTypedDict]
    x11: NotRequired[X11TypedDict]
    zeromq: NotRequired[ZeromqTypedDict]


class Service(BaseModel):
    activemq: Optional[Activemq] = None

    amqp: Optional[Amqp] = None

    any_connect: Optional[AnyConnect] = None

    asterisk_manager_interface: Optional[AsteriskManagerInterface] = None

    bacnet: Optional[Bacnet] = None

    banner: Optional[str] = None

    banner_hash_sha256: Optional[str] = None

    banner_hex: Optional[str] = None

    cert: Optional[Certificate] = None

    checkpoint_topology: Optional[CheckpointTopology] = None

    chromecast: Optional[Chromecast] = None

    cisco_ipsla: Optional[CiscoIpsla] = None

    cmore: Optional[Cmore] = None

    coap: Optional[Coap] = None

    crestron_cp3: Optional[CrestronCp3] = None

    crestron_din_ap2: Optional[CrestronDinAp2] = None

    cwmp: Optional[Cwmp] = None

    darkcomet: Optional[Darkcomet] = None

    darkgate: Optional[Darkgate] = None

    dcerpc: Optional[Dcerpc] = None

    dhcpdiscover: Optional[Dhcpdiscover] = None

    dnp3: Optional[Dnp3] = None

    dns: Optional[DNS] = None

    dtls: Optional[Dtls] = None

    dvr_ip: Optional[DvrIP] = None

    eip: Optional[Eip] = None

    elf_file: Optional[ElfFile] = None

    endpoints: OptionalNullable[List[EndpointScanState]] = UNSET

    epmd: Optional[Epmd] = None

    etcd: Optional[Etcd] = None

    ethereum: Optional[Ethereum] = None

    exposures: OptionalNullable[List[Risk]] = UNSET

    fox: Optional[Fox] = None

    ftp: Optional[Ftp] = None

    gearman: Optional[Gearman] = None

    hardware: OptionalNullable[List[Attribute]] = UNSET

    hid_vertx: Optional[HidVertx] = None

    hikvision: Optional[Hikvision] = None

    ibmnje: Optional[Ibmnje] = None

    ike: Optional[Ike] = None

    imap: Optional[Imap] = None

    iota: Optional[Iota] = None

    ip: Optional[str] = None

    ipmi: Optional[Ipmi] = None

    ipp: Optional[Ipp] = None

    iscsi: Optional[Iscsi] = None

    ja4tscan: Optional[Ja4TScanScan] = None

    jarm: Optional[JarmScan] = None

    krpc: Optional[Krpc] = None

    l2tp: Optional[L2Tp] = None

    labels: OptionalNullable[List[Label]] = UNSET

    ldap: Optional[Ldap] = None

    lpd: Optional[Lpd] = None

    mdns: Optional[Mdns] = None

    memcached: Optional[Memcached] = None

    mikrotik_winbox: Optional[MikrotikWinbox] = None

    minecraft: Optional[Minecraft] = None

    misconfigs: OptionalNullable[List[Risk]] = UNSET

    mms: Optional[Mms] = None

    modbus: Optional[Modbus] = None

    monero_p2p: Optional[MoneroP2P] = None

    mongodb: Optional[Mongodb] = None

    mqtt: Optional[Mqtt] = None

    mssql: Optional[Mssql] = None

    murmur: Optional[Murmur] = None

    mysql: Optional[Mysql] = None

    nats_io: Optional[NatsIo] = None

    nbd: Optional[Nbd] = None

    nfs_mountd: Optional[NfsMountd] = None

    nmea: Optional[Nmea] = None

    ntp: Optional[Ntp] = None

    ntrip: Optional[Ntrip] = None

    onc: Optional[Onc] = None

    onvif: Optional[Onvif] = None

    opc_ua: Optional[OpcUa] = None

    openvpn: Optional[Openvpn] = None

    openvpn_mgmt: Optional[OpenvpnMgmt] = None

    operating_systems: OptionalNullable[List[Attribute]] = UNSET

    oracle: Optional[Oracle] = None

    pc_anywhere: Optional[PcAnywhere] = None

    pgbouncer: Optional[Pgbouncer] = None

    pop3: Optional[Pop3] = None

    port: Optional[int] = None

    portmap: Optional[Portmap] = None

    postgres: Optional[Postgres] = None

    pptp: Optional[Pptp] = None

    profinet_cm: Optional[ProfinetCm] = None

    protocol: Optional[str] = None

    rdate: Optional[Rdate] = None

    rdp: Optional[Rdp] = None

    realport: Optional[Realport] = None

    redis: Optional[Redis] = None

    redline: Optional[Redline] = None

    redlion_crimson: Optional[RedlionCrimson] = None

    representative_info: Optional[RepresentativeInfo] = None

    rifatron: Optional[Rifatron] = None

    ripple: Optional[Ripple] = None

    rlogin: Optional[Rlogin] = None

    rocketmq: Optional[Rocketmq] = None

    rtsp: Optional[Rtsp] = None

    s7: Optional[S7] = None

    sap_router: Optional[SapRouter] = None

    scan_time: Optional[str] = None

    scpi: Optional[Scpi] = None

    screenshots: OptionalNullable[List[Screenshot]] = UNSET

    ser2net: Optional[Ser2Net] = None

    seven_days_to_die: Optional[SevenDaysToDie] = None

    sip: Optional[Sip] = None

    skinny: Optional[Skinny] = None

    smb: Optional[Smb] = None

    smtp: Optional[SMTP] = None

    snmp: Optional[Snmp] = None

    socks: Optional[Socks] = None

    software: OptionalNullable[List[Attribute]] = UNSET

    spice: Optional[Spice] = None

    ssdp: Optional[Ssdp] = None

    ssh: Optional[SSH] = None

    steam: Optional[Steam] = None

    tacacs_plus: Optional[TacacsPlus] = None

    team_viewer: Optional[TeamViewer] = None

    telnet: Optional[Telnet] = None

    threats: OptionalNullable[List[Threat]] = UNSET

    tibia: Optional[Tibia] = None

    tls: Optional[TLS] = None

    tplink_kasa: Optional[TplinkKasa] = None

    transport_protocol: Optional[ServiceTransportProtocol] = None

    unitronics_pcom: Optional[UnitronicsPcom] = None

    upnp: Optional[Upnp] = None

    ventrilo: Optional[Ventrilo] = None

    vnc: Optional[Vnc] = None

    vulns: OptionalNullable[List[Vuln]] = UNSET

    weblogic_t3: Optional[WeblogicT3] = None

    winrm: Optional[Winrm] = None

    ws_discovery: Optional[WsDiscovery] = None

    x11: Optional[X11] = None

    zeromq: Optional[Zeromq] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "activemq",
            "amqp",
            "any_connect",
            "asterisk_manager_interface",
            "bacnet",
            "banner",
            "banner_hash_sha256",
            "banner_hex",
            "cert",
            "checkpoint_topology",
            "chromecast",
            "cisco_ipsla",
            "cmore",
            "coap",
            "crestron_cp3",
            "crestron_din_ap2",
            "cwmp",
            "darkcomet",
            "darkgate",
            "dcerpc",
            "dhcpdiscover",
            "dnp3",
            "dns",
            "dtls",
            "dvr_ip",
            "eip",
            "elf_file",
            "endpoints",
            "epmd",
            "etcd",
            "ethereum",
            "exposures",
            "fox",
            "ftp",
            "gearman",
            "hardware",
            "hid_vertx",
            "hikvision",
            "ibmnje",
            "ike",
            "imap",
            "iota",
            "ip",
            "ipmi",
            "ipp",
            "iscsi",
            "ja4tscan",
            "jarm",
            "krpc",
            "l2tp",
            "labels",
            "ldap",
            "lpd",
            "mdns",
            "memcached",
            "mikrotik_winbox",
            "minecraft",
            "misconfigs",
            "mms",
            "modbus",
            "monero_p2p",
            "mongodb",
            "mqtt",
            "mssql",
            "murmur",
            "mysql",
            "nats_io",
            "nbd",
            "nfs_mountd",
            "nmea",
            "ntp",
            "ntrip",
            "onc",
            "onvif",
            "opc_ua",
            "openvpn",
            "openvpn_mgmt",
            "operating_systems",
            "oracle",
            "pc_anywhere",
            "pgbouncer",
            "pop3",
            "port",
            "portmap",
            "postgres",
            "pptp",
            "profinet_cm",
            "protocol",
            "rdate",
            "rdp",
            "realport",
            "redis",
            "redline",
            "redlion_crimson",
            "representative_info",
            "rifatron",
            "ripple",
            "rlogin",
            "rocketmq",
            "rtsp",
            "s7",
            "sap_router",
            "scan_time",
            "scpi",
            "screenshots",
            "ser2net",
            "seven_days_to_die",
            "sip",
            "skinny",
            "smb",
            "smtp",
            "snmp",
            "socks",
            "software",
            "spice",
            "ssdp",
            "ssh",
            "steam",
            "tacacs_plus",
            "team_viewer",
            "telnet",
            "threats",
            "tibia",
            "tls",
            "tplink_kasa",
            "transport_protocol",
            "unitronics_pcom",
            "upnp",
            "ventrilo",
            "vnc",
            "vulns",
            "weblogic_t3",
            "winrm",
            "ws_discovery",
            "x11",
            "zeromq",
        ]
        nullable_fields = [
            "endpoints",
            "exposures",
            "hardware",
            "labels",
            "misconfigs",
            "operating_systems",
            "screenshots",
            "software",
            "threats",
            "vulns",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from censys_platform.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class RloginTypedDict(TypedDict):
    error: NotRequired[str]
    operating_system: NotRequired[str]
    software: NotRequired[str]
    software_version: NotRequired[str]


class Rlogin(BaseModel):
    error: Optional[str] = None

    operating_system: Optional[str] = None

    software: Optional[str] = None

    software_version: Optional[str] = None
